@@ -18,9 +18,9 @@
 """
 AttchmentField
 """
-__version__ = "$Revision: 23006 $"
+__version__ = "$Revision: 25863 $"
 # $Source: /cvsroot/ingeniweb/PloneSubscription/SubscriptionTool.py,v $
-# $Id: AttachmentHandler.py 23006 2006-05-02 20:25:43Z encolpe $
+# $Id: AttachmentHandler.py 25863 2006-07-07 14:47:14Z manuco $
 __docformat__ = 'restructuredtext'
 
 
@@ -74,19 +74,22 @@ def registerHandler(handler_class):
     # Instanciate the class
     handler = handler_class()        # We instanciate it
 
-    # Compute the converter and previewer paths
-    index_path = handler.getIndexPath(None, None)
-    preview_path = handler.getPreviewPath(None, None)
-    if index_path:
-        handler.index_program = getConverterProgram(
-            handler.converter_type,
-            index_path,
+    if handler.is_external_conv:
+        # Compute the converter and previewer paths
+        index_path = handler.getIndexPath(None, None)
+        preview_path = handler.getPreviewPath(None, None)
+        if index_path:
+            handler.index_program = getConverterProgram(
+                handler
             )
-    if preview_path:
-        handler.preview_program = getConverterProgram(
-            handler.converter_type,
-            preview_path,
+        if preview_path:
+            handler.preview_program = getConverterProgram(
+                handler
             )
+    else:
+        handler.index_program = None
+        handler.preview_program = None
+
 
     # Store the class
     class_name = handler_class.__name__
@@ -112,7 +115,7 @@ def getAttachmentHandler(contentType, field, instance):
     # No match. Have to return the dummy handler
     Log(LOG_DEBUG, "No match, we fall back to the dummy handler", dummy)
     return dummy
-        
+
 
 
 class AbstractHandler(Implicit, ):
@@ -138,6 +141,8 @@ class AbstractHandler(Implicit, ):
         "small_icon_file",                      # (string) of the SMALL icon name IN A PLONE SKIN.
         "content_types",                        # List (of strings) of content_types supported by the class
 
+        "is_external_conv",                     # is the converter using an external programm ?
+        "is_working",                           # is the converter producing anything meaningfull ?
         "index_path",                           # Index program relative path
         "index_arguments",                      # Converter program args. File name will be '%s'.
         "index_encoding",                       # Encoding managed by the encoder (output only - input is nonsence as it's most likely binary data!) or None
@@ -155,6 +160,7 @@ class AbstractHandler(Implicit, ):
     small_icon_file = INVALID_VALUE
     content_types = INVALID_VALUE               # Supported content-types (tuple of strings)
 
+    is_external_conv = INVALID_VALUE
     index_path = INVALID_VALUE
     index_arguments = INVALID_VALUE
     index_encoding = INVALID_VALUE
@@ -164,6 +170,9 @@ class AbstractHandler(Implicit, ):
     preview_encoding = INVALID_VALUE
     preview_format = INVALID_VALUE
 
+    program_found = False
+    error = False ## if true, gui will display it.
+
 
     __CHECK_INTERFACE__ = 1             # Special attribute to enforce IF checking
 
@@ -210,7 +219,7 @@ class AbstractHandler(Implicit, ):
 
     def getIndexableValue(self, field, instance):
         """
-        getIndexableValue(self, field, instance) => (possibliy big) string 
+        getIndexableValue(self, field, instance) => (possibliy big) string
         Return the ZCatalog-indexable string for that type.
         """
         Log(LOG_DEBUG, "converting field", field.getName(), self.index_arguments, self.__class__.__name__)
@@ -307,14 +316,14 @@ class AbstractHandler(Implicit, ):
             f = open(fn, "w+b")
             f.write(content)
             f.close()
-        
+
             # Call converter in the right directory
             Log(LOG_DEBUG, "Calling converter for", filename, "in directory", tempfile.tempdir)
             index = self.callConverter(
                 program,
-                arguments % (fn, ), 
+                arguments % (fn, ),
                 )
-            
+
         finally:
             # Delete attachment file
             if os.path.isfile(fn):
@@ -327,7 +336,7 @@ class AbstractHandler(Implicit, ):
             if type(encoding) in (type(''), type(u''), ):
                 Log(LOG_DEBUG, "Encoding", encoding)
                 index = index.decode(encoding, "replace")
-                
+
             elif type(encoding) in (type(None),):
                 pass
 
@@ -337,14 +346,14 @@ class AbstractHandler(Implicit, ):
                         Log(LOG_DEBUG, "Trying encoding", enc)
                         index = index.decode(enc, "replace")
                         break
-                    
+
                     except UnicodeError:
                         Log(LOG_DEBUG, "Encoding", enc, "failed.")
                         pass
 
         # Return the string
         return index
-        
+
 
     #                                                                   #
     #                         HTML PREVIEW SUPPORT                      #
@@ -356,7 +365,7 @@ class AbstractHandler(Implicit, ):
     _has_body_end = re.compile(r"""</\s*body""", re.I)
     _strip_body_end = re.compile(r"""</\s*body\s*>.*""", re.I | re.S)
     _strip_tags = re.compile(r"""<[^>]+>""", re.I | re.S)
-    
+
 
     def _html_to_text(self, html):
         """crudely convert html to text"""
@@ -374,7 +383,7 @@ class AbstractHandler(Implicit, ):
         # Regular file size
         Log(LOG_DEBUG, "Stripping style...")
         text = self._strip_style.sub('', text, )
-        
+
         # _strip_body_start regexp may go in infinite loop
         # if there is no body attribute (in certain conditions)
         if self._has_body_start.search(text) is not None:
@@ -416,7 +425,7 @@ class AbstractHandler(Implicit, ):
         # Check if we can preview
         if self.preview_arguments is None:
             return None
-        
+
         # Call the converter with the proper arguments
         preview = self._convert(
             field,
@@ -426,7 +435,7 @@ class AbstractHandler(Implicit, ):
             self.preview_encoding,
             )
         Log(LOG_DEBUG, "Getting preview for file", field.getFilename(instance))
-    
+
         # Return the previewable string
         return self._convertOutput(preview, self.preview_format)
 
@@ -457,7 +466,7 @@ class AbstractHandler(Implicit, ):
         """
         callConverter(self, program_path, arguments = '', report_errors = 1) => convert file using program_path with given arguments.
         Return the output stream of the converter program.
-        
+
         if stdin is given, it is feed into the program. Else, it is ignored.
         if report_errors is true, 2> ~/tempfile is appended at the end of the command line
         """
@@ -478,7 +487,7 @@ class AbstractHandler(Implicit, ):
             cmd = "%s 2> %s" % (cmd, errfile, )
         else:
             errfile = None
-        
+
         # Actually execute command
         errors = ""
         curdir = os.getcwd()
@@ -488,7 +497,7 @@ class AbstractHandler(Implicit, ):
             Log(LOG_DEBUG, "We work in", os.getcwd())
             r = os.popen(cmd, "r")
             idx = r.read()
-            
+
         finally:
             # Go back the the current dir
             os.chdir(curdir)
@@ -502,7 +511,7 @@ class AbstractHandler(Implicit, ):
                 else:
                     errors = f.read()
                     f.close()
-                    
+
             # Remove the temporary directory
             try:
                 for root, dirs, files in os.walk(tmpdir, topdown=False):
@@ -511,7 +520,7 @@ class AbstractHandler(Implicit, ):
                     for name in dirs:
                         os.rmdir(os.path.join(root, name))
                 os.rmdir(tmpdir)
-                
+
             except:
                 LogException()
                 Log(LOG_NOTICE, "Could not remove temporary stuff in", tmpdir)
@@ -552,6 +561,32 @@ class AbstractHandler(Implicit, ):
         text = string.strip(text)
         return text
 
+
+    def getGUIIndexProgramCommand(self, field, instance):
+        if not self.is_working:
+            return "<em>unavailable</em>"
+        if not self.is_external_conv:
+            return "<em>internal</em>"
+        elif self.program_found == False:
+            return self.index_path + " " + self.index_arguments + "<br/><strong>not found</strong>"
+        else:
+            return self.index_path + " " + self.index_arguments
+
+    def getGUIPreviewProgramCommand(self, field, instance):
+        if not self.is_working:
+            return "<em>unavailable</em>"
+        if not self.is_external_conv:
+            return "<em>internal</em>"
+        elif self.program_found == False:
+            return self.preview_path + " " + self.preview_arguments + "<br/><strong>not found</strong>"
+        else:
+            return self.preview_path + " " + self.preview_arguments
+
+    def getError(self, field, instance):
+        if self.error:
+            return "error"
+        return ""
+
 #                                                                       #
 #                       External programs interface                     #
 #                                                                       #
@@ -559,15 +594,20 @@ class AbstractHandler(Implicit, ):
 PACKAGE_HOME = App.Common.package_home(globals())
 if sys.platform == 'win32':
     # Windows platform
-    def getConverterProgram(conv_type, conv_path):
+    def getConverterProgram(converter):
+        conv_type = converter.converter_type
+        conv_path = converter.index_path
+        is_external_conv = converter.is_external_conv
+
         if conv_path is None:
             return None
         if conv_path.lower() == "type":
             return "type"
 
-        program = '"' + os.path.join(PACKAGE_HOME, "converters", conv_type, "win32", conv_path, ) + '"' 
+        program = '"' + os.path.join(PACKAGE_HOME, "converters", conv_type, "win32", conv_path, ) + '"'
         if os.path.isfile(program[1:-1]):
             Log(LOG_DEBUG, "Using '%s' program to convert %s attachments." % (program, conv_type))
+            converter.program_found = True
         else:
             Log(
                 LOG_WARNING,
@@ -576,16 +616,24 @@ if sys.platform == 'win32':
                 ),
                 )
             program = None
-            
+            converter.error = True
+
         return program
 
 else:
-    def getConverterProgram(conv_type, conv_path):
+    def getConverterProgram(converter):
+        conv_type = converter.converter_type
+        conv_path = converter.index_path
+        is_external_conv = converter.is_external_conv
         # Unix platform
         import commands
-        if conv_path is None:
+        if not is_external_conv:
             return None         # In case we don't need a program
 
+        if "(internal)" == conv_path or conv_path is None:
+            raise RuntimeError("converter path is invalid, but external conv has been set.")
+
+
         # Try to find the real full path of the program
         program = conv_path
         program = commands.getoutput("which %s" % (program))
@@ -596,8 +644,10 @@ else:
                     conv_path, conv_type,
                     ),
                 )
+            converter.error = True
         else:
             program = string.strip(program)
             Log(LOG_DEBUG, "Using '%s' program to convert %s attachments." % (program, conv_type))
-            
+            converter.program_found = True
+
         return program
